#!/usr/bin/perl -w

###########################################################################
# webwatch-fe.pl
#
# Basic proof-of-concept front-end to our webwatch data gatherer.
#
# $Id: webwatch-fe.pl,v 1.1 2020/04/05 03:19:09 pmdg Exp $
###########################################################################

###########################################################################
# Modules
###########################################################################
use strict;
use File::Basename;
use CGI;

###########################################################################
# Globals
###########################################################################
my $query;
my $dataPath = "/Users/pmdg/tmp/webwatch";
my %data = ();


###########################################################################
# Subroutines
###########################################################################

# get_data() retrieves the data for us to use. Return 1 on successful
# retrieval, 0 on failure.
sub get_data {
	my $sentinelWrite = $dataPath . "/writing.lck";
	my $sentinelRead = $dataPath . "/reading.lck";
	
	# Hmmm, we'll probably need to write a reading.lck file as well
	# so that we don't get the files clobbered while we're reading
	# them.
	if (-f $sentinelWrite) {
		sleep(5);
		if (-f $sentinelWrite) {
			sleep(5);
			if (-f $sentinelWrite) {
				sleep (5);
				if (-f $sentinelWrite) {
					print "<B><FONT COLOR=RED>Error: Sentinel file remains present. Try again later></FONT></B>\n";
				}
			}
		}
	}
	
	# Clear out any previous data at this point.
	undef %data;
	my @sessions = ();
	my @volumes = ();
	my @devices = ();
	
	# Write the sentinel file.
	if (open(SENT,">$sentinelRead")) {
		print SENT localtime(time) . "\n";
	}
	close(SENT);
	
	# If we made it to here we should be OK to read.
	my $volumes = $dataPath . "/volumes.data";
	my $sessions = $dataPath . "/sessions.data";
	my $devices = $dataPath . "/devices.data";
	if (open(VOLS,$volumes)) {
		while (<VOLS>) {
			my $line = $_;
			chomp $line;
			push(@volumes,$line);
		}
		close(VOLS);
	}
	if (open(SESS,$sessions)) {
		while (<SESS>) {
			my $line = $_;
			chomp $line;
			push(@sessions,$line);
		}
		close(SESS);
	}
	if (open(DEV,$devices)) {
		while (<DEV>) {
			my $line = $_;
			chomp $line;
			push(@devices,$line);
		}
		close(DEV);
	}
	
	# We should check this.
	unlink($sentinelRead);
	
	# Now process the data.
	$data{sessions}{count} = @sessions+0;
	if (@sessions+0 > 0) {
		my $sessCount = 0;
		foreach my $sess (@sessions) {
			my ($completed,$mode,$saveset,$client,$group,$device,$volume,$startTime,$stopped,$xferMB,$sizeGB) = (split(/\|\|\|/,$sess))[0,1,2,3,4,5,6,7,8,9,10];
			$data{sessions}{session}{$sessCount}{completed} = $completed;
			$data{sessions}{session}{$sessCount}{mode} = $mode;
			$data{sessions}{session}{$sessCount}{client} = $client;
			$data{sessions}{session}{$sessCount}{group} = $group;
			$data{sessions}{session}{$sessCount}{saveset} = $saveset;
			$data{sessions}{session}{$sessCount}{device} = $device;
			$data{sessions}{session}{$sessCount}{volume} = $volume;
			$data{sessions}{session}{$sessCount}{start} = $startTime;
			$data{sessions}{session}{$sessCount}{stopped} = $stopped;
			$data{sessions}{session}{$sessCount}{xferMB} = $xferMB;
			$data{sessions}{session}{$sessCount}{sizeGB} = $sizeGB;
			$sessCount++;
		}
	}
	
	$data{volumes}{count} = @volumes+0;
	if (@volumes+0 > 0) {
		foreach my $vol (@volumes) {
			my ($volID,$volName,$volPool,$mounts,$written,$recovered) = (split(/\|\|\|/,$vol))[0,1,2,3,4,5];
			$data{volumes}{volume}{$volName}{id} = $volID;
			$data{volumes}{volume}{$volName}{pool} = $volPool;
			$data{volumes}{volume}{$volName}{mounts} = $mounts;
			$data{volumes}{volume}{$volName}{written} = $written;
			$data{volumes}{volume}{$volName}{recovered} = $recovered;
		}
	}
	
	$data{devices}{count} = @devices+0;
	if (@devices+0 > 0) {
		foreach my $dev (@devices) {
			my ($devName,$mediaFamily,$mediaType,$readOnly,$status,$volID,$volPool,$mountedVolume,$lastMessage) = (split(/\|\|\|/,$dev))[0,1,2,3,4,5,6,7,8];
			
			$mountedVolume = ($mountedVolume eq "null") ? "None" : $mountedVolume;
			$lastMessage = ($lastMessage eq "null") ? "<I>No activity</I>" : $lastMessage;
			$data{devices}{device}{$devName}{family} = $mediaFamily;
			$data{devices}{device}{$devName}{type} = $mediaType;
			$data{devices}{device}{$devName}{readonly} = $readOnly;
			$data{devices}{device}{$devName}{volid} = $volID;
			$data{devices}{device}{$devName}{pool} = $volPool;
			$data{devices}{device}{$devName}{volume} = $mountedVolume;
			$data{devices}{device}{$devName}{lastmessage} = $lastMessage;
		}
	}
}

# display_data() shows the data
sub display_data {
	print "<DIV class=all>\n";
	my $indent = "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;";
	
	print "<DIV class=sessions>\n";
	print "<H2> Sessions</H2>\n";
	if (%data && defined($data{sessions}) && defined($data{sessions}{count}) &&
		$data{sessions}{count} > 0) {
		print "<TABLE WIDTH=100% BORDER=0>\n";
		foreach my $sessionCount (keys $data{sessions}{session}) {
			my $client = $data{sessions}{session}{$sessionCount}{client};
			my $mode = $data{sessions}{session}{$sessionCount}{mode};
			my $saveset = $data{sessions}{session}{$sessionCount}{saveset};
			my $group = $data{sessions}{session}{$sessionCount}{group};
			my $device = $data{sessions}{session}{$sessionCount}{device};
			my $volume = $data{sessions}{session}{$sessionCount}{volume};
			my $startTime = $data{sessions}{session}{$sessionCount}{start};
			my $xferMB = $data{sessions}{session}{$sessionCount}{xferMB};
			my $sizeGB = $data{sessions}{session}{$sessionCount}{sizeGB};
			print "<TR>\n";
			print "<TD BGCOLOR=#C9C9C9><I>$mode</I></tTD>\n";
			print "<TD BGCOLOR=#C9C9C9><b>$client</b>";
			if ($saveset ne "" && $saveset ne "null") {
				print ":$saveset</TD>\n";
			} else {
				print "</TD>\n";
			}
			print "<TD BGCOLOR=#C9C9C9>$device:$volume</TD>\n";
			print "<TD BGCOLOR=#C9C9C9>$xferMB MB/s</TD>\n";
			print "<TD BGCOLOR=#C9C9C9>$sizeGB GB</TD>\n";
			print "</TR>\n";
			print "<TR>\n";
			print "<TD COLSPAN=3 BGCOLOR=#FAFAFA>$group</TD>\n";
			print "<TD COLSPAN=2 BGCOLOR=#FAFAFA>$startTime</TD>\n";
			print "</TR>\n";
		}
		
		print "</TABLE>\n";
	} else {
		print "<p><B><I>No active sessions</I></B>\n";
	}
	
	print "</DIV>\n"; 	# Sessions
	
	print "<DIV class=devices>\n";
	print "<H2> Devices</H2>\n";
	if (%data && defined($data{devices}) && defined($data{devices}{count}) &&
		$data{devices}{count} > 0) {
		print "<TABLE WIDTH=100% BORDER=0>\n";
		foreach my $deviceName (sort {lc $a cmp lc $b} keys $data{devices}{device}) {
			print "<TR>\n";
			my $family = $data{devices}{device}{$deviceName}{family};
			my $type = $data{devices}{device}{$deviceName}{type};
			my $readonly = $data{devices}{device}{$deviceName}{readonly};
			my $pool = $data{devices}{device}{$deviceName}{pool};
			my $volume = $data{devices}{device}{$deviceName}{volume};
			my $message = $data{devices}{device}{$deviceName}{lastmessage};
			
			my $readOnlyStart = ($readonly) ? "<I>" : "";
			my $readOnlyStop = ($readonly) ? "</I>" : "";
			print "<TD BGCOLOR=#C9C9C9>$readOnlyStart<b>$deviceName</b>$readOnlyStop</TD>\n";
			print "<TD BGCOLOR=#C9C9C9>$type</TD>\n";
			print "<TD BGCOLOR=#C9C9C9>$volume</TD>\n";
			print "<TD BGCOLOR=#C9C9C9>($pool)</TD>\n";
			print "</TR>";
			print "<TR>\n";
			print "<TD COLSPAN=4 BGCOLOR=#FAFAFA>$indent $message</TD>";
			print "</TR>\n";
		}
		print "</TABLE>\n";
	}
	print "</DIV>\n";	# Devices
	
	print "<DIV CLASS=volumes>\n";
	print "<H2>Volumes</H2>\n";
	if (%data && defined($data{volumes}) && defined($data{volumes}{count}) &&
		$data{volumes}{count} > 0) {
		
		print "<TABLE WIDTH=100% BORDER=0>\n";
		print "<TR>\n";
		print "<TD class=title>Volume</TD><TD class=title>Pool</TD><TD class=title>ID</TD><TD class=title>Mounts</TD><TD class=title>Written</TD><TD class=title>Recovered</TD>\n";
		print "</TR>\n";
		my $bgColor = "#C9C9C9";
		
		foreach my $volName (sort {lc $a cmp lc $b} keys $data{volumes}{volume}) {
			print "<TR>\n";
			my $volID = $data{volumes}{volume}{$volName}{id};
			my $mounts = $data{volumes}{volume}{$volName}{mounts};
			my $pool = $data{volumes}{volume}{$volName}{pool};
			my $written = $data{volumes}{volume}{$volName}{written};
			my $recovered = $data{volumes}{volume}{$volName}{recovered};
			print "<TD BGCOLOR=$bgColor><B>$volName</B></TD>\n";
			print "<TD BGCOLOR=$bgColor>$pool</TD>\n";
			print "<TD BGCOLOR=$bgColor>$volID</TD>\n";
			print "<TD BGCOLOR=$bgColor>$mounts</TD>\n";
			print "<TD BGCOLOR=$bgColor>$written</TD>\n";
			print "<TD BGCOLOR=$bgColor>$recovered</TD>\n";
						
			if ($bgColor eq "#C9C9C9") {
				$bgColor = "#FAFAFA";
			} else {
				$bgColor = "#C9C9C9";
			}
			print "</TR>\n";
		}
	}
	print "</DIV>\n";	# Volumes
	print "</DIV>\n";	# All
}


###########################################################################
# Main
###########################################################################

$query = new CGI;
print $query->header(-Refresh => "20;url=/cgi-bin/webwatch-fe.cgi");
print $query->start_html(
		-title => 'Prototype NSR Webwatch',
		-author => 'preston@nsrd.info',
		-style => { 'src' => '/webwatch.css'} 
	);

print "<DIV class=title>Prototype NSR Webwatch (" . localtime(time) . ")\n</div>";

get_data();
display_data();

print $query->end_html();
die "\n";
